<?php

use PHPUnit\Framework\TestCase;

final class InteractiveSortSampleTest extends TestCase
{
    private static $itemsFilepath = __DIR__ . '/../items.txt';
    private static $pid = 0;
    private $sessid = '';

    private function curlRequest($formValue = null)
    {
        $url = 'http://127.0.0.1:8080/index.php';
        $ch = curl_init($url);
        curl_setopt_array($ch, [
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_COOKIESESSION => true,
            CURLOPT_HEADER => true,
        ]);
        if (!is_null($formValue)) {
            curl_setopt_array($ch, [
                CURLOPT_POST => true,
                CURLOPT_POSTFIELDS => ['choice' => $formValue]
            ]);
        }
        if (!empty($this->sessid)) {
            curl_setopt(
                $ch,
                CURLOPT_HTTPHEADER,
                ['Cookie: PHPSESSID=' . $this->sessid]
            );
        }
        $response = curl_exec($ch);
        curl_close($ch);

        if (empty($this->sessid)) {
            preg_match_all('/^Set-Cookie:\s*([^;]*)/mi', $response, $matches);
            $cookies = array();
            foreach ($matches[1] as $item) {
                parse_str($item, $cookie);
                $cookies = array_merge($cookies, $cookie);
            }
            $this->sessid = $cookies['PHPSESSID'];
        }

        return $response;
    }

    private function xpathQuery($query, $html)
    {
        $libxml_use_internal_errors = libxml_use_internal_errors(true);
        if (empty($query) || empty($html)) {
            return new DOMNodeList();
        }
        $xpath = $this->htmlToDomXPath($html);
        $results = $xpath->query($query);
        libxml_use_internal_errors($libxml_use_internal_errors);

        return $results;
    }

    private function xpathEvaluate($expression, $html)
    {
        $libxml_use_internal_errors = libxml_use_internal_errors(true);
        if (empty($expression) || empty($html)) {
            return false;
        }
        $xpath = $this->htmlToDomXPath($html);
        $result = $xpath->evaluate($expression);
        libxml_use_internal_errors($libxml_use_internal_errors);

        return $result;
    }

    private function htmlToDomXPath($html)
    {
        $dom = new DomDocument();
        $dom->loadHTML('<?xml encoding="utf-8"?>' . $html);
        $xpath = new DomXPath($dom);

        return $xpath;
    }

    private function saveItems($items)
    {
        file_put_contents(self::$itemsFilepath, implode("\n", $items));
    }

    public static function setUpBeforeClass(): void
    {
        $op = [];
        exec('nohup php -S 127.0.0.1:8080 -t ' . __DIR__ . '/../ > /dev/null 2>&1 & echo $!', $op);
        sleep(3);
        self::$pid = $op[0];
    }

    public function setUp(): void
    {
        $this->sessid = '';
    }

    public function test1Item()
    {
        $items = [
            'something'
        ];
        $this->saveItems($items);

        $response = $this->curlRequest();
        $this->assertTrue($this->xpathEvaluate('boolean(//p[@id="result"])', $response));
        $result = $this->xpathQuery('//p[@id="result"]', $response)[0]->nodeValue;
        $result = trim($result);
        $this->assertEquals($items[0], $result);
    }

    public function test2Items()
    {
        $items = [
            'item1',
            'item2'
        ];
        $this->saveItems($items);

        $response = $this->curlRequest();
        $this->assertTrue($this->xpathEvaluate('boolean(//p[@id="question"])', $response));
        $this->assertTrue($this->xpathEvaluate('boolean(//div[@id="choice1"])', $response));
        $this->assertTrue($this->xpathEvaluate('boolean(//div[@id="choice2"])', $response));
        $choice1 = $this->xpathQuery('//div[@id="choice1"]', $response)[0]->nodeValue;
        $choice1 = trim($choice1);
        $choice2 = $this->xpathQuery('//div[@id="choice2"]', $response)[0]->nodeValue;
        $choice2 = trim($choice2);
        $question = $this->xpathQuery('//p[@id="question"]', $response)[0]->nodeValue;
        $question = trim($question);
        $this->assertContains($choice1, $items);
        $this->assertContains($choice2, $items);
        $this->assertContains($question, ['item1 or item2?', 'item2 or item1?']);

        $response = $this->curlRequest('1');
        $this->assertTrue($this->xpathEvaluate('boolean(//p[@id="result"])', $response));
        $result = $this->xpathQuery('//p[@id="result"]', $response)[0]->nodeValue;
        $result = trim($result);
        $this->assertEquals('item2 < item1', $result);
    }

    public static function tearDownAfterClass(): void
    {
        exec('kill ' . self::$pid);
        file_put_contents(
            self::$itemsFilepath,
            implode("\n", [
                'PHP course',
                'Python course',
                'Java course'
            ])
        );
    }
}
